#!/usr/bin/env perl

package iANT::Common;

use strict;
use warnings;
use Carp;
use IO::File;
use CGI;

require 'UNIVERSAL.pl';

=head1 NAME

iANT::DB::Feature - Object to deal with Entry's features pooled into one XML file


=head1 SYNOPSIS

    use iANT;

=head1 DESCRIPTION

    iANT package contains a set of methods (Get, Set, Dump)used by the following modules:
    
    iANT::Entry
    iANT::Entry::Definition
    iANT::Entry::Link
    iANT::Entry::Sequence
    iANT::Feature
    iANT::Feature::Database
    iANT::Feature::Software
    iANT::Feature::Sequence
    iANT::Feature::Hit
    iANT::Feature::Hit::Property
    iANT::Feature::Hit::Raw
    iANT::Organism
    iANT::Pattern
    iANT::DB::Feature
    iANT::DB::Fasta

=cut

=pod

=head3 procedure Dump

 Title        : Dump
 Usage        : $o_object->Dump();
 Prerequisite : none
 Function     : Dump an object in xml format
 Returns      : none
 Args         : $target:-optional; if empty or undef, print on STDOUT
                -if defined, print in the specified file (unix-like mode is mandatory)
 Globals      : none

=cut

sub Dump
{
    my ($self, $target) = (@_);
    my $print_file = 0;
    if ((defined $target) && ($target ne ''))
    {
        if ($target !~ /^>/)
        {
            croak("\n\nYou must define the opening file mode: \">\" for overwrite or \">>\" for append mode\n\n");
        }
        else
        {
            open(OUT, "$target") || confess("$!:$target");
            select OUT;
            $print_file = 1;
        }
    }


    my $o_twig = XML::Twig->new();
    $o_twig->set_pretty_print('indented');
    
    my $xml = $self->GetXml();
    
    $o_twig->parse($xml);


    print '<?xml version="1.0"?>', "\n", $o_twig->sprint();

    
	if ($print_file)
    {
        select STDOUT;
        close OUT;
    } 
	
    return;

}

=pod
 
=head3 Getters and Setters

=item Get

    Function     : Generic Getter
    Args         : $attribute

=cut

sub Get
{
    my ($self, $attribute) = (@_);
    if ((defined $attribute) && ($attribute ne ''))
    {
        return $self->{'__' . $attribute} if (defined $self->{'__' . $attribute});
    }
    else
    {
        confess("Get method needs an attribute name\n");
    }
}

=pod

=item Set

    Function    : Generic Setter
    Args        : $attribute
                  $value

=cut

sub Set
{
    my ($self, $attribute, $value) = @_;
    if ((defined $attribute) && ($attribute ne ''))
    {
        $self->{'__' . $attribute} = $value if (defined $self->{'__' . $attribute});
    }
    else
    {
        confess("Set method needs an attribute name\n");
    }
    return;

}

=pod

=item Backup

    Function    : Clone original object

=cut

sub Backup
{
    my $self = shift();
    if (!defined $self->{__o_backup})
    {
        my $o_clone = UNIVERSAL::Clone($self);
        $self->{'__o_backup'} = $o_clone;
    }
    return;

}

=pod

=item Restore

    Function    : Restore original object

=cut

sub Restore
{
    my $self = shift();
    if (defined $self->{'__o_backup'} && (ref($self->{'__o_backup'}) eq ref($self)))
    {

        #my $class   = ref($self);
        #my $o_original = bless {%{$self->{'__o_backup'}}}, $class;
        #return $o_original;
        %{$self} = %{$self->{'__o_backup'}};
    }
    else
    {
        confess("No backup available\n");
    }

}

=pod

=item NumericTest

    Function    : Do a numeric test from a string
    Args        : $left value
                  $operator (>,>=,<,<=,=) : empty operator is considered as '='
			   $right value
    Returns     : boolean

=cut

sub NumericTest
{
    my ($left, $operator, $right) = (@_);

    my $test_result = 0;

    if ($operator eq '>')
    {
        $test_result = 1 if ($left > $right);
    }
    elsif ($operator eq '>=')
    {
        $test_result = 1 if ($left >= $right);
    }
    elsif ($operator eq '<')
    {
        $test_result = 1 if ($left < $right);
    }
    elsif ($operator eq '<=')
    {
        $test_result = 1 if ($left <= $right);
    }
    elsif (($operator eq '=') || ($operator eq ''))
    {
        $test_result = 1 if ($left == $right);
    }
    return $test_result;
}

=pod

=item IsMatchingSelectionParameter

    Function    : Scan an object (iANT::Entry, iANT::Feature, ...) and
                  test if it is matching parameters (element/attributes)
    Args        : $o_scanned_object: the object to be scanned
                  $rh_selection_parameters: a hash ref of selection parameters 
    Returns     : boolean

=cut

sub IsMatchingSelectionParameter
{
    my ($o_scanned_object, $rh_selection_parameters) = (@_);

    my $match_selection_parameters = 0;
    foreach my $selection_parameter (keys %{$rh_selection_parameters})
    {
        if (ref($rh_selection_parameters->{$selection_parameter}) =~ /hash/i)
        {
            my $o_parameter_element = $o_scanned_object->Get('o_' . $selection_parameter);
            if (defined $o_parameter_element && ($o_parameter_element ne ''))
            {
                foreach my $element_selection_parameter (keys %{$rh_selection_parameters->{$selection_parameter}})
                {
                    if ($rh_selection_parameters->{$selection_parameter}->{$element_selection_parameter} =~
                        /^([\<\>=]+)(\d+)$/)
                    {
                        if (&NumericTest($o_parameter_element->Get($element_selection_parameter), $1, $2))
                        {
                            $match_selection_parameters = 1;
                        }
                        else
                        {
                            $match_selection_parameters = 0;
                            last;
                        }
                    }

                    elsif ($o_parameter_element->Get($element_selection_parameter) =~
                           /^$rh_selection_parameters->{$selection_parameter}->{$element_selection_parameter}$/)
                    {
                        $match_selection_parameters = 1;
                    }
                    else
                    {
                        $match_selection_parameters = 0;
                        last;
                    }
                }
                if (!$match_selection_parameters)
                {
                    last;
                }
            }
            else
            {
                my $ra_o_parameter_element = $o_scanned_object->Get('ra_' . $selection_parameter . 's');
                if (defined $ra_o_parameter_element && (ref($ra_o_parameter_element) =~ /array/i))
                {
                    foreach my $o_parameter_element (@{$ra_o_parameter_element})
                    {
                        foreach
                          my $element_selection_parameter (keys %{$rh_selection_parameters->{$selection_parameter}})
                        {

                            if ($rh_selection_parameters->{$selection_parameter}->{$element_selection_parameter} =~
                                /^([\<\>=]+)(\d+)$/)
                            {
                                if (&NumericTest($o_parameter_element->Get($element_selection_parameter), $1, $2))
                                {
                                    $match_selection_parameters = 1;
                                }
                                else
                                {
                                    $match_selection_parameters = 0;
                                    last;
                                }
                            }
                            elsif ($o_parameter_element->Get($element_selection_parameter) =~
                                   /^$rh_selection_parameters->{$selection_parameter}->{$element_selection_parameter}$/)
                            {
                                $match_selection_parameters = 1;
                            }
                            else
                            {
                                $match_selection_parameters = 0;
                                last;
                            }
                        }
                        if (!$match_selection_parameters)
                        {
                            last;
                        }
                    }
                }
            }
        }
        else
        {
            if ($rh_selection_parameters->{$selection_parameter} =~ /^([\<\>=]+)(\d+)$/)
            {
                if (&NumericTest($o_scanned_object->Get($selection_parameter), $1, $2))
                {
                    $match_selection_parameters = 1;
                }
                else
                {
                    $match_selection_parameters = 0;
                    last;
                }
            }
            elsif ($o_scanned_object->Get($selection_parameter) =~ /^$rh_selection_parameters->{$selection_parameter}$/)
            {
                $match_selection_parameters = 1;
            }
            else
            {
                $match_selection_parameters = 0;
                last;
            }
        }
    }
    return $match_selection_parameters;
}

=pod

=item function MultiFastaFile2DBEntry

 Title        : MultiFastaFile2DBEntry
 Usage        : MultiFastaFile2DBEntry($fasta_file,$o_param)
 Prerequisite : fasta2entrydb defined in $o_param
 Function     : Converts a multifasta file into an iANT::DB::Entry object
                using iANT.aa_dna.FastaEntryDBConverter.pl program
 Returns      : $o_db_entry (iANT::DB::Entry)
 Args         : $fasta_file the multifasta file path
                $o_param ParamParser object
 Globals      : none
 Note	    : TO BE OPTIMIZED ...
 
=cut

sub MultiFastaFile2DBEntry
{
    my ($fasta_file, $o_param) = (@_);

    if (!$o_param->IsDefined('fasta2entrydb'))
    {
        confess("fasta2entrydb not defined ...\n");
        return 0;
    }
    else
    {
        my $fasta2entrydb_cmd =
          $o_param->Get('fasta2entrydb') . ' --sequence ' . $fasta_file ;
	$fasta2entrydb_cmd .= ' --type ' . $o_param->Get('type') if ($o_param->IsDefined('type'));

        print STDERR "\n" . time() . "\tLaunch $fasta2entrydb_cmd\n" if ($o_param->IsDefined('verbose'));
        my $entry_db_xml = `$fasta2entrydb_cmd`;
        print STDERR "\n" . time() . "\tBuild DB::Entry\n" if ($o_param->IsDefined('verbose'));
        my $o_db_entry = New iANT::DB::Entry($entry_db_xml);
        print STDERR "\n" . time() . "\tDB::Entry [OK]\n" if ($o_param->IsDefined('verbose'));

        return $o_db_entry;
    }
}

=pod

=item function WhateverToiANTEntryList

 Title        : WhateverToiANTEntryList
 Usage        : WhateverToiANTEntryList($input,$o_param)
 Function     : From every type of input (directory, fof,file - entry xml, db::entry xml, fasta -, accession)
			 try to build an array of iANT::Entry objects;
			 Test Sequence is: 
				1. directory 
				2. file
					a. Entry
					b. DB::Entry
					c. Fasta
					d. file of file (recursive call of this function in that case)
				3. accession
 Returns      : $ra_o_entry (reference to a list of iANT::Entry)
 Args         : $input
                $o_param ParamParser object
 Globals      : [conf] extension_iant_entry , extension_iant_db_entry, unix_find
 
=cut

sub WhateverToiANTEntryList
{
    my ($input, $o_param) = (@_);

    my @a_o_entry = ();
$o_param = &iANT::Cfg::Load($o_param);
    

    my $extension_iant_entry = $o_param->Get('extension_iant_entry');
    my $unix_find            = $o_param->Get('unix_find');    
    my $extension_iant_db_entry = $o_param->Get('extension_iant_db_entry');
        
    if (&IsDir($input))
    {

        my @a_entry_files        = `$unix_find $input -type f -name \\*.$extension_iant_entry`;
        chomp @a_entry_files;
        foreach my $entry_file (@a_entry_files)
        {
            my $o_entry = New iANT::Entry($entry_file);
            $o_entry->Set('xmlfile', $entry_file);
            push(@a_o_entry, $o_entry);
        }

        my @a_db_entry_files        = `$unix_find $input -type f -name \\*.$extension_iant_db_entry`;
        chomp @a_db_entry_files;
        foreach my $db_entry_file (@a_db_entry_files)
        {
            my $o_db_entry = New iANT::DB::Entry($db_entry_file);
            foreach my $o_entry (@{$o_db_entry->GetEntries()})
            {
                $o_entry->Set('xmlfile', $db_entry_file);
                push(@a_o_entry, $o_entry);
            }
        }
    }
    elsif (-e $input)
    {
        if (&IsiANTEntry($input))
        {
            my $o_entry = New iANT::Entry($input);
            $o_entry->Set('xmlfile', $input);
            push(@a_o_entry, $o_entry);
        }
        elsif (&IsiANTDBEntry($input))
        {
	
            my $o_db_entry = New iANT::DB::Entry($input);
            foreach my $o_entry (@{$o_db_entry->GetEntries()})
            {
#                $o_entry->Set('xmlfile', $input);
                push(@a_o_entry, $o_entry);
            }
        }
        elsif (&IsFasta($input))
        {
            my $o_db_entry = MultiFastaFile2DBEntry($input, $o_param);
            foreach my $o_entry (@{$o_db_entry->GetEntries()})
            {
                if (! defined $o_entry->Get('xmlfile') || ($o_entry->Get('xmlfile') eq ''))
                {
                    $o_entry->Set('xmlfile',  $input . '.' . $o_entry->Get('accession') . '.' . $extension_iant_entry);
                }
                push(@a_o_entry, $o_entry);
            }

        }
        elsif (@a_o_entry = @{&IsFof($input, $o_param, 'WhateverToiANTEntryList')})
        {
            1;
        }

    }
    else
    {
        if (my $entry_file = &IsAccession($input, $o_param))
        {
            my $o_entry = New iANT::Entry($entry_file);
            $o_entry->Set('xmlfile', $entry_file);
            push(@a_o_entry, $o_entry);
        }
        else
        {
            confess("Unknown input >$input<");
        }
    }

    return \@a_o_entry;
}

=pod

=item function IsAccession

 Title        : IsAccession
 Usage        : IsAccession($input,$o_param)
 Function     : Scan project data_dir to find an entry file with $input as an accession
 Returns      : $entry_file (entry xml file) if  found
			 0 if not found
 Args         : $input
                $o_param ParamParser object
 Globals      : [conf] data_dir
 
=cut

sub IsAccession
{
    my ($input, $o_param) = (@_);
    if ($o_param->IsDefined('data_dir'))
    {
        if (my $entry_file = &__FindiANTEntry($o_param->Get('data_dir'), $input, $o_param))
        {
            return $entry_file;
        }
        else
        {
            die(">$input< entry file cannot be found in " . $o_param->Get("data_dir") . " directory");
            return 0;
        }
    }
    else
    {
        die("If >$input< is an accession code, you must define the >data_dir< parameter to find this entry");
        return 0;
    }

}

=pod

=item function __FindiANTEntry

 Title        : __FindiANTEntry
 Usage        : __FindiANTEntry($scanned_drectory,$accession,$o_param)
 Function     : Scan a directory to find an entry file with $accession as an accession
			 using unix find command.
			 First, look for an entry directory named $accession; then look into this
			 directory if an entry file exist and is a real iANT::Entry file.
 Returns      : $entry_file (entry xml file) if  found
			 0 if not found
 Args         : $scanned_directory
			 $accession
                $o_param ParamParser object
 Globals      : [conf] unix_find, extension_iant_entry
 
=cut

sub __FindiANTEntry
{
    my ($scanned_dir, $accession, $o_param) = (@_);

    my $unix_find = $o_param->Get('unix_find');

    my $cmd           = "$unix_find $scanned_dir -type d -follow -name $accession";
    my @a_directories = `$cmd`;
    chomp @a_directories;

    if ($#a_directories > 0)
    {
        confess("More than one directory matching with $accession:\n" . join("\n", @a_directories));
    }
    my $accession_directory = $a_directories[0];

    if (defined $accession_directory && ($accession_directory ne ''))
    {
        my $entry_file = "$accession_directory/$accession" . '.' . $o_param->Get('extension_iant_entry');
        if ((-e $entry_file) && &iANT::Common::IsiANTEntry($entry_file))
        {
            return $entry_file;
        }
        else
        {
            return 0;
        }
    }
    return 0;

}

=pod

=item function IsDir

 Title        : IsDir
 Usage        : IsDir($input)
 Function     : Test if $input is a directory
 Returns      : boolean
 Args         : $input
 Globals      : none
 
=cut

sub IsDir
{
    my $input = shift();
    if (-d $input)
    {
        return 1;
    }
    else
    {
        return 0;
    }
}

=pod

=item function IsFastaGeneric

 Title        : IsFastaGeneric
 Usage        : IsFastaGeneric($input,$letters)
 Function     : Test if $input is a fasta file (has a header and a sequence string) 
 Returns      : boolean
 Args         : $input
			 $letters (regular expression of allowed letters in seuence string)
 Globals      : none
 
=cut

sub IsFastaGeneric
{
    my $input   = shift();
    my $letters = shift();
    $letters = '\w+' if ((!defined $letters) || ($letters eq ''));

    
    my $o_file = new IO::File;
    if ($o_file->open("< $input"))
    {
    my $header   = 0;
        my $sequence = 0;
        while (my $line = <$o_file>)
        {
            
		chomp $line;
	    
        if ($line ne '')
            {
                if ($line =~ /^>\S+/)
                {
                    $header = 1;
			
                }
                elsif ($line =~ /^$letters$/i)
                {
                    $sequence = 1;
                }
                else
                {
                    return 0;
                }
            }
            
        }
        if ($header && $sequence)
        {
            return 1;
        }
        else
        {
            return 0;
        }
        $o_file->close;

    }
    else
    {
        return 0;
    }
}

=pod

=item function IsFasta

 Title        : IsFasta
 Usage        : IsFasta($input)
 Function     : Test if $input is a fasta file (has a header and a sequence string) 
 Returns      : boolean
 Args         : $input
 Globals      : none
 
=cut

sub IsFasta
{
    my $input = shift();
    my $result = &IsFastaGeneric($input, '[\w\s]+');
    return $result;
}

=pod

=item function IsFastaNucleic

 Title        : IsFastaNucleic
 Usage        : IsFastaNucleic($input)
 Function     : Test if $input is a nucleic fasta file (has a header and a sequence string
			 containing only ATCGNU letters) 
 Returns      : boolean
 Args         : $input
 Globals      : none
 
=cut

sub IsFastaNucleic
{
    my $input = shift();
    my $result = &IsFastaGeneric($input, '[atcgun]+');
    return $result;
}

=pod

=item IsFof

 Function     	:   Test if an input is a file of files
 Args          : $input - tested data
                 $o_param - ParamParser object
		       $source_sub - the function to apply on a file
 Returns		: $ra_list: a list reference of what is returned by $source_sub
 Globals		: none

=cut

sub IsFof
{
    my ($input, $o_param, $source_sub) = (@_);
    my $ra_list = [];
    my @a_files = ();
    my $o_file  = new IO::File("$input");
    return 0 if (!defined $o_file);

    while (my $line = <$o_file>)
    {
        chomp $line;
        push(@a_files, $line);

    }
    $o_file->close;

    foreach my $file (@a_files)
    {
        if (-e $file)
        {
            print STDERR "$file\n";
            my $new_ra_list = eval("&$source_sub('$file','$o_param')");
            if ($@ ne '')
            {
                print STDERR "ERROR  - $@";
            }
            else
            {
                push(@{$ra_list}, @{$new_ra_list});
            }
        }
    }

    return $ra_list;
}

sub GetFileNamesFromFof
{
    my ($input) = (@_);
    my @a_files = ();
    my $o_file  = new IO::File("$input");
    return 0 if (! -e $input);

    while (my $line = <$o_file>)
    {
        chomp $line;
	return 0 if (! -e $line);
        push(@a_files, $line);

    }
    $o_file->close;
    return \@a_files;
}

=pod

=item function IsiANTEntry

 Title        : IsiANTEntry
 Usage        : IsiANTEntry($input)
 Function     : Test if $input is an Entry Xml file 
			 (test root node name and try to build a iANT::Entry object) 
 Returns      : boolean
 Args         : $input
 Globals      : none
 
=cut

sub IsiANTEntry
{
    my $input = shift();
    eval 'use iANT::Entry';
    return 0 if ($@);

    my $o_entry;

    if (! -e $input)
    {
    	my $tmp_file = "/tmp/$$" .$^T;
    	my $o_file = new IO::File(">$tmp_file");
	print $o_file $input;
	$o_file->close;
	$input = $tmp_file;
    }


    my $o_file = new IO::File($input);
    return 0 if (!defined $o_file);
    while (my $line = <$o_file>)
    {
        chomp $line;
        if ($line =~ /^<\?.+\?>\s*$/)
        {
            next;
        }
        else
        {
            if ($line =~ /<entry\s/)
            {
                eval {$o_entry = New iANT::Entry($input)};
                if ($@)
                {
                    return 0;
                }
                else
                {
                    return $o_entry;
                }
            }
            else
            {
                return 0;
            }
        }
    }
    $o_file->close;
}

=pod

=item function IsiANTDBEntry

 Title        : IsiANTDBEntry
 Usage        : IsiANTDBEntry($input)
 Function     : Test if $input is a DB::Entry Xml file 
			 (test root node name and try to build a iANT::DB::Entry object) 
 Returns      : boolean
 Args         : $input
 Globals      : none
 
=cut

sub IsiANTDBEntry
{
    my $input = shift();
    eval 'use iANT::DB::Entry';
    return 0 if ($@);
    my $o_db_entry;
    

    if (($input =~ /\n/) || (! -e $input))
    {
    	my $tmp_file = "/tmp/$$" .$^T;
    	my $o_file = new IO::File(">$tmp_file");
	print $o_file $input;
	$o_file->close;
	$input = $tmp_file;
    }

    my $o_file = new IO::File($input);
    return 0 if (!defined $o_file);
    while (my $line = <$o_file>)
    {

        chomp $line;
        if ($line =~ /^<\?.+\?>\s*$/)
        {
            next;
        }
        else
        {
            if ($line =~ /<entry_db\s*/)
            {
                eval {$o_db_entry = New iANT::DB::Entry($input)};
                if ($@)
                {
                    $o_file->close;
                    return 0;
                }
                else
                {
                    $o_file->close;
                    return $o_db_entry;
                }
            }
            else
            {
                $o_file->close;
                return 0;
            }
        }
    }
    $o_file->close;
}

=pod

=item function IsiANTDBFeature

 Title        : IsiANTDBFeature
 Usage        : IsiANTDBFeature($input)
 Function     : Test if $input is a DB::Feature Xml file 
			 (test root node name and try to build a iANT::DB::Feature object) 
 Returns      : boolean
 Args         : $input
 Globals      : none
 
=cut

sub IsiANTDBFeature
{
    my $input = shift();
    eval 'use iANT::DB::Feature';
    return 0 if ($@);
    my $o_db_feature;
   
if (! -e $input)
    {
    	my $tmp_file = "/tmp/$$" .$^T;
    	my $o_file = new IO::File(">$tmp_file");
	print $o_file $input;
	$o_file->close;
	$input = $tmp_file;
    }

    my $o_file = new IO::File($input);
    return 0 if (!defined $o_file);
    while (my $line = <$o_file>)
    {
        chomp $line;
        if ($line =~ /^<\?.+\?>\s*$/)
        {
            next;
        }
        else
        {
            if ($line =~ /<entry_features\s?/)
            {
                eval {$o_db_feature = New iANT::DB::Feature($input)};
                if ($@)
                {
                    return 0;
                }
                else
                {
                    return $o_db_feature;
                }
            }
            else
            {
                return 0;
            }
        }
    }
    $o_file->close;
}

=pod

=item function GetDBFeatureFile

 Title        : GetDBFeatureFile
 Usage        : GetDBFeatureFile($input, $o_param)
 Function     : Look for a DB::Feature xml file from a Entry or DB::Entry file name
 Returns      : $entry_db_feature_file file name if found
			 else undef
 Args         : $input (an entry file name, an entry db file name)
			 $o_param
 Globals      : [conf] extension_iant_entry, extension_iant_db_entry, extension_iant_db_feature
 
=cut

sub GetDBFeatureFile
{
    my ($input, $o_param) = (@_);
    my $entry_db_feature_file           = $input;
    my $entry_file_extension            = $o_param->Get('extension_iant_entry');
    my $entry_db_file_extension         = $o_param->Get('extension_iant_db_entry');
    my $entry_db_feature_file_extension = $o_param->Get('extension_iant_db_feature');

    $entry_db_feature_file =~ s/$entry_file_extension$//;
    $entry_db_feature_file =~ s/$entry_db_file_extension$//;
    $entry_db_feature_file .= $entry_db_feature_file_extension;

    if (-e $entry_db_feature_file)
    {
        return $entry_db_feature_file;
    }
    return;

}

=pod

=item function BuildHtmlSelect

 Title        : BuildHtmlSelect
 Usage        : BuildHtmlSelect($data, $input_name, $select)
 Function     : Build an HTML popup menu from different kind of source:
				1. a file: value|label\nvalue|label\n...
				2. an array: (value|label,value|label...)
				3. a hash: (value => lable, value =>label, ...)
				4. a string: value|label#value|label#...
 Returns      : $popup_menu html code
 Args         : $data
			 $input_name name of html input
			 $select default selected item
 Globals      : none
 
=cut

sub BuildHtmlSelect
{
    my ($data, $input_name, $select) = (@_);

    my %h_select_data;

    if (defined $data && ($data ne ''))
    {
        if (-e $data)
        {
            my $o_file = new IO::File;
            if ($o_file->open("< $data"))
            {
                while (my $line = <$o_file>)
                {
                    chomp $line;
                    my ($value, $label) = split(/\|/, $line, 2);
                    if (!defined $label)
                    {
                        $label = $value;
                    }
                    $h_select_data{$value} = $label;
                }
                $o_file->close;
            }
        }
        elsif (ref($data) =~ /array/i)
        {
            foreach my $item (@{$data})
            {
                chomp $item;
                my ($value, $label) = split(/\|/, $item, 2);
                if (!defined $label)
                {
                    $label = $value;
                }
                $h_select_data{$value} = $label;
            }
        }
        elsif (ref($data) =~ /hash/i)
        {
            %h_select_data = %{$data};
        }
        else
        {
            my @a_fields = split(/#/, $data);
            foreach my $item (@a_fields)
            {
                chomp $item;
                my ($value, $label) = split(/\|/, $item, 2);
                if (!defined $label)
                {
                    $label = $value;
                }
                $h_select_data{$value} = $label;
            }
        }

        my $o_cgi = New CGI();
        my $popup_menu = $o_cgi->popup_menu(
                                            -name    => $input_name,
                                            -default => $select,
                                            -values  => [sort (keys %h_select_data)],
                                            -labels  => \%h_select_data
                                            );

        return $popup_menu;
    }
    else
    {
        return;
    }

}

=pod

=item function GetValue

 Title        : GetValue
 Usage        : GetValue($data, $select)
 Function     : Get label associated to a value from different kind of source 
				1. a file: value|label\nvalue|label\n...
				2. an array: (value|label,value|label...)
				3. a hash: (value => lable, value =>label, ...)
				4. a string: value|label#value|label#...
 Returns      : $label
 Args         : $data
			 $select selected item
 Globals      : none
 
=cut

sub GetValue
{
    my ($data, $select) = (@_);

    my %h_select_data;

    if (defined $data && ($data ne ''))
    {
        if (-e $data)
        {
            my $o_file = new IO::File;
            if ($o_file->open("< $data"))
            {
                while (my $line = <$o_file>)
                {
                    chomp $line;
                    my ($value, $label) = split(/\|/, $line, 2);
                    if (!defined $label)
                    {
                        $label = $value;
                    }
                    $h_select_data{$value} = $label;
                }
                $o_file->close;
            }
        }
        elsif (ref($data) =~ /array/i)
        {
            foreach my $item (@{$data})
            {
                chomp $item;
                my ($value, $label) = split(/\|/, $item, 2);
                if (!defined $label)
                {
                    $label = $value;
                }
                $h_select_data{$value} = $label;
            }
        }
        elsif (ref($data) =~ /hash/i)
        {
            %h_select_data = %{$data};
        }
        else
        {
            my @a_fields = split(/#/, $data);
            foreach my $item (@a_fields)
            {
                chomp $item;
                my ($value, $label) = split(/\|/, $item, 2);
                if (!defined $label)
                {
                    $label = $value;
                }
                $h_select_data{$value} = $label;
            }
        }

        return $h_select_data{$select};
    }
    else
    {
        return;
    }

}



sub BuildFastaHeader
{
	
    my $param_entry = shift();
    my $o_param = shift();
    
    $o_param = New ParamParser('INIT') if (! defined $o_param);
    my $id        = '';
    my $accession = '';
    my $len       = '';
    my $species   = '';
    my $type      = '';
    my $def       = '';
    my $pm        = '';
    my $chr       = '';

    if (ref($param_entry) =~ /entry/i)
    {
        my $o_entry = $param_entry;
        $id        = $o_entry->Get('id');
        $accession = $o_entry->Get('accession');
        $len       = $o_entry->Get('length');
        $species   = $o_entry->GetOrganism->GetSpecies->GetCode();
        $type      = $o_entry->Get('type');

        my $ra_o_definition = $o_entry->GetDefinitions();
        my @a_pmid          = ();

        foreach my $o_definition (@{$ra_o_definition})
        {
            if ($o_definition->GetOntology() eq 'pubmed')
            {
                push(@a_pmid, $o_definition->GetId());
            }
            elsif (($o_definition->GetOntology() eq 'NONE') && ($o_definition->GetId() eq 'product'))
            {
                $def = $o_definition->GetContent();
            }
        }
        $pm = join(';', @a_pmid);

        my $ra_o_link = $o_entry->GetLinks();
        my @a_contigs = ();

        foreach my $o_link (@{$ra_o_link})
        {
            if ($o_link->GetType() eq 'partOf')
            {
                push(@a_contigs, $o_link->GetAccession());
            }
        }

        $chr = join(';', @a_contigs);
    }
    elsif (ref($param_entry) =~ /hash/i)
    {
        my $rh_sequence_info = $param_entry;
        $id        = $rh_sequence_info->{'id'}        if (defined $rh_sequence_info->{'id'});
        $accession = $rh_sequence_info->{'accession'} if (defined $rh_sequence_info->{'accession'});
        $len       = $rh_sequence_info->{'length'}    if (defined $rh_sequence_info->{'length'});
        $species   = $rh_sequence_info->{'species'}   if (defined $rh_sequence_info->{'species'});
        $type      = $rh_sequence_info->{'type'}      if (defined $rh_sequence_info->{'type'});
        my @a_pmid = ();

        foreach my $rh_definition (@{$rh_sequence_info->{'ra_definitions'}})
        {
            if ($rh_definition->{'ontology'} eq 'pubmed')
            {
                push(@a_pmid, $rh_definition->{'id'});
            }
            elsif (($rh_definition->{'ontology'} eq 'NONE') && ($rh_definition->{'id'} eq 'product'))
            {
                $def = $rh_definition->{'content'};
            }
        }
        $pm = join(';', @a_pmid);
        my @a_contigs = ();
        foreach my $rh_link (@{$rh_sequence_info->{'ra_links'}})
        {
            if ($rh_link->{'type'} eq 'partOf')
            {
                push(@a_contigs, $rh_link->{'accession'});
            }
        }
        $chr = join(';', @a_contigs);
    }

    #>id acc=sp:P15711 len=87 chr=1 sp=code_prj k=[EBVA] pm=1689460;1689461 type=gene|cds|protein def=description

    my %h_known_database = (
                            'gb'   => 'gb|accession|locus',
                            'emb'  => 'emb|accession|locus',
                            'dbj'  => 'dbj|accession|locus',
                            'pir'  => 'pir||entry',
                            'prf'  => 'prf||name',
                            'sp'   => 'sp|accession|entry name',
                            'pdb'  => 'pdb|entry|chain',
                            'pat'  => 'pat|country|number',
                            'bbs'  => 'bbs|number',
                            'gnl'  => 'gnl|database|identifier',
                            'ref'  => 'ref|accession|locus',
                            'lcl'  => 'lcl|identifier',
                            'none' => 'identifier'
                            );
    my $fasta_header = '>';
    my $id_syntax    = '';
    
    my $database     = $o_param->Get('database');
    if (!$o_param->IsDefined('database'))
    {

        #cas ou le id est deja formate
        foreach my $known_database (keys %h_known_database)
        {
            if ($id =~ /^$known_database\|/)
            {
                $id_syntax = $h_known_database{'none'};
                last;
            }
        }
        $id_syntax = $h_known_database{'lcl'} if ($id_syntax eq '');
    }
    elsif (!defined $h_known_database{$database})
    {
        $id_syntax = $h_known_database{'lcl'};
    }
    else
    {
        $id_syntax = $h_known_database{$database};
    }

    $id_syntax =~ s/accession/$accession/;
    $id_syntax =~ s/locus/$id/;
    $id_syntax =~ s/identifier/$id/;
    $id_syntax =~ s/entry name/$id/;
    $id_syntax =~ s/entry/$accession/;
    $id_syntax =~ s/database/$database/;

    if ($type eq '' && $o_param->IsDefined('type'))
    {
        $type = $o_param->Get('type');
    }

    $o_param->Set('type', $type);

    $fasta_header .= $id_syntax;
    $fasta_header .= " acc=$accession";
    $fasta_header .= " len=$len";
    $fasta_header .= " sp=$species";
    $fasta_header .= " type=$type";
    $fasta_header .= " chr=$chr";
    $fasta_header .= " pm=$pm";
    $fasta_header .= " def=$def";

    return $fasta_header;

}

sub FormatSequenceLine
{
    my ($sequence, $length_line) = (@_);

    $sequence =~ s/\?/X/g;
    $sequence =~ s/\W//g;
    #if (&__ControlSequenceType($sequence))
    #{
#    $sequence =~ s/\s//g;

        my ($size_sequence) = length($sequence);
        my ($nb_lines)      = $size_sequence / $length_line;
        my ($j)             = 0;
        $nb_lines = int($nb_lines) + 1 if ($nb_lines =~ /\./);
        my $formatted_sequence = '';
        for ($j = 0; $j < $nb_lines; $j++)
        {
            $formatted_sequence .= substr($sequence, ($j) * $length_line, $length_line) . "\n";
        }
        return $formatted_sequence;
    #}

}


1;
